System Platforms Workflow Compare ⬇ Download Free
NEW NVIDIA Jetson support is live — SSH, tegrastats, CUDA deploy Download →
AI IDE FOR REAL HARDWARE

Code.
Deploy.
Monitor.

One AI IDE built for software that runs on real hardware. Jetson · Android · Arduino · Embedded Linux · Automotive · Robotics.

Other AI editors stop when the code is written. X02 keeps going — SSH into the board, cross-compile, flash the MCU, watch the GPU, read the serial line. One window. No blind deploys.

6+
Platforms
CUDA
Native Support
7
AI Providers
95MB
IDE Size
35ms
Surgical Edits
🟢 Jetson Orin · 74% GPU
📱 Pixel 7 · Mirror Active
🔧 ESP32-S3 · Flashed OK
Operator X02 — inference.cu
CUDA Jetson
PROJECT
inference.cu
MainActivity.kt
sensor.ino
api/auth.ts
train.py
🟢 inference.cu
📱 MainActivity.kt
1// CUDA inference kernel — sm_87 Ampere
2__global__ void infer(float* input, float* out) {
3  int idx = blockIdx.x * blockDim.x + threadIdx.x;
4  if (idx < N) out[idx] = relu(input[idx]);
5}
6
7// X02 AI → deploy to Jetson Orin
8infer<<<blocks, threads>>>(d_input, d_out);
9cudaDeviceSynchronize();
10// ✓ Compiled — binary uploaded to /home/nvidia/
 GPU 74%
|
58°C
|
CUDA · Ln 8
✓ Deployed
🟢 NVIDIA Jetson · Edge AI
📱 Android · Kotlin · ADB
🔧 Arduino · ESP32 · STM32
🛡️ MISRA C · ISO 26262
🌐 React · Python · Rust · 40+ Frameworks
🧠 AI / ML · PyTorch · ONNX · TensorRT
Surgical Edit Engine · 35ms
🔒 100% Local · No Cloud · No Login
🟢 NVIDIA Jetson · Edge AI
📱 Android · Kotlin · ADB
🔧 Arduino · ESP32 · STM32
🛡️ MISRA C · ISO 26262
🌐 React · Python · Rust · 40+ Frameworks
🧠 AI / ML · PyTorch · ONNX · TensorRT
Surgical Edit Engine · 35ms
🔒 100% Local · No Cloud · No Login
// one workspace, one system

Build entire intelligent systems.
Not just applications.

Software. Hardware. AI. Cloud. Every node below is written, built, deployed and monitored from the same window — and they talk to each other.

Not "supports Android" — Android talks to your ESP32. Not "supports Jetson" — Jetson deploys models from the same workspace. Not "supports Arduino" — Arduino streams sensor data back into the editor.
READY Watch the data move, or click any node to see what X02 does there.

Every device in this diagram is a folder in the same project. One AI context. One deploy panel. One monitor.

// the 5-second version

Cursor writes your code.
X02 ships it to the board.

Cursor · VS Code General purpose

Excellent at writing web and application code.

  • AI autocomplete & chat
  • Repo-wide context
  • ~ SSH to a board extension
  • ~ Flash firmware extension
  • ~ Serial monitor extension
  • GPU / thermal telemetry
  • Android device mirror
  • Deploy → run → monitor loop
Operator X02 Hardware native

Purpose-built for embedded, robotics and edge AI development.

  • AI autocomplete & chat 7 providers
  • Repo-wide context boundless memory
  • SSH to a board built in
  • Flash firmware arduino · esp32 · stm32
  • Serial monitor + plotter
  • GPU / thermal telemetry tegrastats
  • Android device mirror scrcpy
  • Deploy → run → monitor loop
// built for
Embedded engineers Robotics developers Automotive software Edge AI engineers Jetson developers Android developers Arduino makers
6+
Supported Platforms
40+
Build Frameworks
7
AI Providers
172+
AI Memory Signals
Free
MIT License Forever
GitHub Stars
Release Downloads
Current Version
Last Updated
MIT
License · Forever
// what it actually builds

Not "40+ frameworks."
These, by name.

CUDA Jetson Orin / Nano TensorRT PyTorch ONNX Python Arduino ESP32 STM32 RPi Pico Embedded Linux Kotlin Jetpack Compose Gradle C / C++ MISRA C Rust React Next.js Qt Node.js Tauri

Toolchains are auto-detected by the Setup Wizard. If it isn't installed, X02 tells you exactly what's missing.

// supported platforms

Built for the hardware
other IDEs ignore.

Every platform gets first-class treatment — AI autocomplete, one-click deploy, live hardware monitoring, and Boundless Memory context across all your sessions.

📱
Android
Kotlin · Java · Gradle · ADB
Build, compile, and screen-mirror your Android device in real time. AI-assisted Kotlin/Java with live device preview — no Android Studio bloat.
Screen mirror via scrcpy — 60fps live
Gradle build with streaming output
APK install & launch one click
ADB device management built-in
$ x02 android --build --deploy
Gradle: BUILD SUCCESSFUL in 12s
APK installed → Pixel 7 (USB)
Screen mirror active — 60fps
Kotlin Java Gradle scrcpy
🔧
Arduino / Embedded
C/C++ · PlatformIO · Serial
Flash firmware and monitor serial output without leaving your editor. Auto-detect boards, AI embedded C/C++, and a built-in serial plotter for sensor data.
Auto board detect — Arduino, ESP32, STM32
One-click compile & flash
Serial monitor & plotter built-in
MISRA C safety hints
$ x02 flash --board esp32-s3
Compiling sketch... 98% flash
Flashed OK → /dev/ttyUSB0
Serial: temp=23.4°C hum=61%
Arduino ESP32 STM32 MISRA C
🛡️
Safety-Critical
MISRA C · ISO 26262 · DO-178C
Automotive and aerospace firmware with built-in compliance checks. AI understands safety standards and warns you before violations ship.
MISRA C:2012 rule checking
ISO 26262 ASIL-aware AI suggestions
Traceability & doc helpers
Static analysis integration
AI ⚠ Rule 11.3 violation (ln 42)
Cast: pointer ↔ integer type
→ Fix applied (0 violations)
MISRA C ISO 26262 AUTOSAR DO-178C
🌐
Web / Fullstack
React · Python · Rust · 40+ frameworks
Full web and backend development with AI autocomplete across 40+ frameworks. Build system auto-detection, SVN/Git VCS, and the Surgical Edit Engine for ~35ms precision edits.
40+ frameworks — React, Next.js, Vue, Vite, Django…
Surgical Edit Engine — 8-stage, ~35ms
SVN & Git VCS awareness
Multi-AI: Claude, GPT, Gemini, Groq…
AI Applied 3 surgical edits to auth.ts
Backup → .x02-backup/
0 destructive changes — safe ✓
React Python Rust TypeScript
🧠
AI / ML Pipelines
PyTorch · TensorFlow · ONNX
Train on workstation, deploy to edge. Camera-based code analysis, autonomous multi-file coding, and Boundless Memory with 172+ trigger patterns for massive AI context.
Boundless Memory — 172+ context triggers
Camera panel — vision API code capture
Autonomous coding — multi-file edits
ONNX → TensorRT pipeline templates
AI Boundless Memory: 172 triggers loaded
Autonomous edit: refactored 8 files
All tests passing — 0 regressions ✓
PyTorch ONNX Vision API Edge AI
// see it in action

Watch X02
do the work.

Real deploys. Real hardware. No cuts — CUDA on Jetson, Kotlin on Android, C++ on Arduino. All from one IDE.

📱 ANDROID DEVELOPMENT
Build, deploy & device mirror walkthrough — Kotlin + ADB + scrcpy in X02
🔧 ARDUINO / EMBEDDED
Arduino sketch compile, flash & serial monitor — live from X02 IDE
// developer workflow

Code → Deploy → Monitor
in one window.

No context switching. No blind deploys. No separate terminal for monitoring. Everything lives in X02.

✍️
Write
AI autocomplete + Surgical Edits with backup
7 AI providers
🚀
Deploy
One-click to any target — Jetson, Android, MCU
SSH · ADB · USB
⚙️
Build
Compile on device with live output streaming
nvcc · Gradle · make
▶️
Run
Execute and stream results in integrated terminal
Live output
📊
Monitor
Live hardware metrics, temps, power and alerts
tegrastats · ADB
one real task · optimise a CUDA kernel on a Jetson Orin
01
Write the kernel
Monaco with native CUDA definitions. AI completes against your whole project.
inference.cu · sm_87
02
Compile
nvcc runs on the device, output streams back into the IDE terminal.
nvcc -arch=sm_87 inference.cu
03
Upload over SSH
Key-based SSH built in. No separate client, no scp muscle memory.
→ nvidia@jetson:/home/nvidia/
04
Run on target
Execute on real silicon, not a simulator. Live stdout in the same window.
./infer --batch 32
05
Watch the GPU
tegrastats parsed into live gauges — utilisation, temperature, power draw.
GPU 74% · 58°C · 12.4W
06 · AI
Find the bottleneck
The AI reads the kernel and the telemetry together, and names the stall.
"uncoalesced global reads"
07 · AI
Optimise
Surgical edit applied to the exact lines, with a backup and a diff you approve.
~35ms · 8-stage pipeline
08
Deploy
Re-run and compare. GPU 74% → 41%. Same window, no alt-tab, whole loop.
✓ deployed · 1.8× faster
// why x02 exists

What you get that
others don't ship.

Most AI IDEs stop the moment the code is written. That is fine if your program's final destination is a browser tab.

Embedded developers have a second half to the job. You still need to SSH into a board, cross-compile, flash a microcontroller, watch a GPU's temperature under load, inspect a serial line at 115200 baud, and debug a device sitting on the bench next to you. Today that means an editor, a terminal, a flashing tool, a monitor, and a lot of alt-tabbing.

Operator X02 brings the complete engineering workflow into one IDE — an embedded AI IDE rather than another AI code editor. Same AI you already expect, plus the hardware half nobody else ships.

// compare the workflow, not the feature list
Workflow step Operator X02 Cursor VS Code Android Studio
AI coding 7 providers~ Copilot~ Gemini
Deploy to Jetson SSH + CUDA~ Manual
Monitor GPU / thermals tegrastats
Mirror an Android device scrcpy~ Extension~ Emulator
Flash a microcontroller Built-in~ Extension
Read the serial line + plotter~ Extension
Safety-critical hints MISRA C
Do it all with no cloud Fully offline Cloud
Open the full feature matrix — memory footprint, CUDA support, price
Feature Operator X02 VS Code Cursor Arduino IDE
Jetson GPU monitoring
tegrastats, SSH, live gauges
Built-in
Android screen mirror
scrcpy integration, ADB
scrcpy
Arduino / embedded flash
serial monitor + plotter
Built-in ~ Extension
AI autocomplete
multi-provider, calibrated
7 providers ~ Copilot
CUDA language support
Monaco native definitions
Native ~ Extension ~ Extension
Surgical AI code edits
8-stage Rust pipeline, ~35ms
8-stage engine ~ Basic
Safety-critical (MISRA C)
ISO 26262, DO-178C hints
Built-in
Persistent AI memory
172+ triggers, local, no cloud
Boundless ~ Basic
100% local / offline
no telemetry, no account
Fully offline Cloud
Open source (MIT)
fork, modify, own it
MIT ~ Partially
IDE memory footprint
Rust/Tauri backend
95 MB ~500 MB ~600 MB ~200 MB
Price Free Forever Free (Copilot: $10/mo) $20/mo Free
// common questions

Before you ask.
We answered it.

Windows and Linux. The IDE is built on Tauri + Rust and ships as a Windows installer plus .deb / .rpm packages. macOS is on the roadmap. Target platforms — Jetson (Linux), Android, Arduino — are supported regardless of which OS the IDE runs on.

No account. No login. No telemetry. Every core feature — build, deploy, flash, monitor, SSH — runs fully offline. AI features require an API key from your chosen provider (Claude, OpenAI, Gemini, Groq, DeepSeek), or use Ollama for 100% local AI with no internet at all.

Yes — free forever under the MIT License. Fork it, modify it, ship it. Built as a solo passion project, not a business. There are no plans for paywalled features, subscriptions, or VC funding. Bring your own API keys and you pay only the AI provider's rates.

The IDE itself is one 95 MB install. Platform toolchains (Android SDK, Arduino CLI, CUDA) are auto-detected or guided by the built-in Setup Wizard. X02 tells you exactly what's missing and installs it — no manual PATH editing or doc-diving required.

Boundless Memory uses 172+ local trigger patterns to automatically surface relevant context — file types, board names, error patterns, build outputs — without cloud sync or manual tagging. Everything stays on-device. Cursor's memory is basic and requires cloud; X02 is deeply embedded-aware and runs offline.

Yes. X02 supports 7 AI providers: Claude, OpenAI, Gemini, Groq, DeepSeek, and Ollama. Ollama lets you run models like Llama 3, Mistral, and CodeLlama entirely on your local machine — no API costs, no internet dependency.

Free & Open Source

Stop switching tools.
Start shipping.

Operator X02 is free, open-source under MIT, and built by a solo developer who believes coding is art. No login. No telemetry. No subscription. Bring your own API keys and build on real hardware.

MIT License · Windows & Linux · 95 MB · No account · Works offline